#!/usr/bin/env python
# -*- coding: utf-8 -*-

__version__ = '1.1'
__date__= '31.01.2014'
import sys
import os
import cgi
import urllib
import subprocess
from traceback import format_exc
cjson = None;  json = None
try:
    import cjson
except ImportError:
    pass
if not cjson:
    try:
        import json
    except ImportError:
        pass
if not cjson and not json: 
    try:
        import simplejson as json
    except ImportError:
        raise ImportError('You must have the cjson, json, or simplejson module available.')
def jdumps(obj, encoding='utf-8'):
    if cjson:
        return cjson.encode(obj)
    else:
        return json.dumps(obj, encoding=encoding)
def jloads(json_string):
    if cjson:
        return cjson.decode(json_string)
    else:
        return json.loads(json_string)
import string
import random
def random_id(length=8):
    choices = string.lowercase+string.digits
    return_id = ''
    for i in range(length):
        return_id += random.choice(choices)
    return return_id
class CallError(Exception):
    def __init__(self, jpcError):
        Exception.__init__(self, jpcError)
        self.error = jpcError
class Dictbool(dict,object):
    def __init__(self, val, *ar):
        super(Dictbool,self).__init__(*ar)
        self['result']=val
    def __nonzero__(self):
        return self.get('result',False)
    def __repr__(self):
        return self.get('message','')
class BaseAuth:
    def __init__(self, j):
        self.users={}
        if isinstance(j, dict):
            self.users = j
        elif isinstance(j, list):
            try:
              for u in j:
                   self.users[u['usern']]=u
            except StandardError:
                pass
    def authproc(self):
        def auth(j):
            try:
                un=j['usern']
                if un in  self.users:
                    u=self.users[un]
                else:
                    return Dictbool(False,{'message':'Authentication user unknown'})
                if 'passwd' in u and 'passwd' in j:
                    if u['passwd']==j['passwd']:
                        return Dictbool(True,{'message':'Authentication success'})
                    else:
                        return Dictbool(False,{'message':'Password mismatch'})
                elif 'token' in u and 'token' in j:
                    if u['token']==j['token']:
                        return Dictbool(True,{'message':'Authentication success'})
                    else:
                        return Dictbool(False,{'message':'Password mismatch'})
                else:
                    return Dictbool(False,{'message':'Authentication element missing'})
            except StandardError:
                pass
            return Dictbool(False,{'message':'Authentication error'})
        return auth
class Client:
    """ modified version of ServiceProxy from jsonprc module
        this class provides the abstraction layer for calling JPC functions as
        ``normal'' functions.
        Instance of this class is a ``function'' and have ``methods'' i.e.:
            funct = Client(('tuple of', 'names'), 'URL or other address', 'access method')
            print funct({'any': 'json'})
        and
            print funct.other('first par', 'sec par')
        where 'other' is the last element of name, i.e name tuple is: ('tuple of', 'names', 'other')
        if there is only one argument it is used as input JSON,
        if only positional arguments, they are collected as list,
        if only keywords arguments, the are collected into dictionary
        if both input JSON is a dict with two keys 'ar' and 'kw' which meanings are guessable
        modification of jsonrpc class was made for  a tuple as method name
        and different method of funtion access:
            htpp -- by urllib
            proc -- by subprocess
            fstr -- call function with JSON dump (string)
            func -- function call
    """
    def __init__(self, Name=None, Call=None, Method=None, Auth=None):
        if not Method in ('http', 'proc', 'fstr', 'func', ):
            if not Method:
                if Call and Call.find('http'):
                    Method = 'http'
                else:
                    raise ValueError('Call method must have value')
            if not isinstance(Method, str):
                raise TypeError('Call method have to be a string')
            Method = Method.lower()
            if Method in ('h', 'http', 'url', ): Method = 'http'
            elif Method in ('p', 'proc', 'subprocess', 'process', ): Method = 'proc'
            elif Method in ('fs', 'fstr', ): Method = 'fstr'
            elif Method in ('f', 'func', 'function', ): Method = 'func'
            else:
                raise ValueError('Unknown call method')
        if not Call:
            raise ValueError('None call address')
        if isinstance(Name, str):
            Name = (Name,)
        elif isinstance(Name, (list, tuple)):
            Name = tuple(Name)
        else:
            raise ValueError('Call name have to be string, tuple or list')
        self.__serviceCall = Call
        self.__serviceMethod = Method
        self.__serviceName = Name
        self.__serviceAuth = Auth
    def __getattr__(self, name):
        serviceName = list(self.__serviceName)
        if isinstance(name, str):
            serviceName.append(name)
        elif isinstance(name, (tuple, list)):
            serviceName.extend(name)
        else:
            raise KeyError('Service subclassing not found')
        return Client(Name=tuple(serviceName), Call=self.__serviceCall, Method=self.__serviceMethod, Auth=self.__serviceAuth)
    def __call__(self, *ar, **kw):
        if len(ar) == 1:
            params = ar[0]
        elif len(ar) == 0:
            if len(kw) == 0:
                params = None
            else:
                params = kw
        else: # lens(ar)>1
            if len(kw) == 0:
                params = ar
            else:
                params = {'ar': ar, 'kw': kw}
        postobj = {"method": self.__serviceName, 'params': params, 'id': random_id(), 'auth': self.__serviceAuth}
        if self.__serviceMethod == 'func':
            respobj = self.__serviceCall(postobj)
        else:
            poststr = jdumps(postobj)
            if self.__serviceMethod == 'fstr':
                respstr = self.__serviceCall(poststr)
            elif self.__serviceMethod == 'proc':
                proc = subprocess.Popen(self.__serviceCall, shell=False, \
                    stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
                respstr, resperror = proc.communicate(poststr)
                if resperror:
                     raise CallError(resperror)
            elif self.__serviceMethod == 'http':
                httppars = urllib.urlencode({'json': poststr})
                respstr = urllib.urlopen(self.__serviceCall, httppars).read()
                if respstr.startswith(('<h','<?','<!')):
                    respstr=respstr.replace('"','\\"').replace('\n',' ')
                    respstr='{"error": {"code": "%s", "message": "%s"}}' % ('', respstr)
            respobj = jloads(respstr)
        if 'error' in respobj and respobj['error'] != None:
            raise CallError(respobj['error'])
        else:
            return respobj['result']
    service = __getattr__
    call = __call__
def set_error(js={}, code=-1, mess=''):
    js['error'] = {'code': code, 'message': mess}
    return js
class Server:
    """ this class provides the ``server'' of JPC functions
        main functionality is assign function (or redirection to other ``servers'') under
        ``extended'', hierarchical names (tuples of names)
        second functionality is to provide ``access'' methods to itself:
          JPC -- a function to be called with a JSON object, and acts as server
          JPCS -- a function to be called with a JSON string, and behaves as previos
          JPC_filter -- a procedure to make server ``filter'' from stdin to stdout
          JPC_CGI -- CGI handler for JPC to be called by HTTP
        and
          JPC, JPCS = main(__name__) -- which make use of preceding four functions
    """
    def __init__(self, tests=False, Auth=None, Logger=None):
         self.procedures = {}
         if Auth:
            self.auth = Auth
         else:
            self.auth = lambda j: True # everybody invited
         if Logger:
            self.logg = Logger
         else:
            self.logg = lambda j: None # no saved logs
         if tests:
            self.add_service('echo', lambda x: x)
            def null(j):
                if isinstance(j, list): return []
                if isinstance(j, dict): return {}
                return None
            self.add_service('null', null)
         self.logg({'event': "server initialised"})
    def find_name(self, name):
        p = self.procedures
        if isinstance(name, str):
            if name not in p:
                p[name] = {}
            p = p[name]
        elif isinstance(name, (tuple, list)):
            for n in name:
                if n not in p:
                    p[n] = {}
                p = p[n]
        else:
            raise ValueError('Bad service ident')
        return p
    def add_service(self, name, func=None, Auth=None, Logger=None):
        p = self.find_name(name)
        p['__call__'] = func
        p['__name__'] = name
        if Auth:
            p['__auth__'] =  Auth
        else:
            p['__auth__'] =  self.auth
        if Logger:
            p['__logg__'] =  Logger
        else:
            p['__logg__'] =  self.logg
        self.logg({'event': "service added", 'value': name})
    def add_proxy(self, name, Name=None, Call=None, Method=None, Auth=None, Logger=None):
        p = self.find_name(name)
        p['__proxy__'] = Client(Name=Name, Call=Call, Method=Method)
        p['__name__'] = name
        if Auth:
            p['__auth__'] =  Auth
        else:
            p['__auth__'] =  self.auth
        if Logger:
            p['__logg__'] =  Logger
        else:
            p['__logg__'] =  self.logg
        self.logg({'event': "proxy service added", 'value': name})
    def JPC(self, inp):
        out = {}
        if not isinstance(inp, dict) or not ('method' in inp):
            return set_error(out, -32601, u'Invalid JPC input')
        logg = self.logg
        def call_error(num, mess):
            set_error(out, num, mess)
            logg({'event': "call failed", 'value': mess})
        met = inp['method']
        if 'id' in inp:
            out['id'] = inp['id']
        if 'params' in inp:
            par = inp['params']
        else:
            par = None
        if isinstance(met, basestring):
            met = [met]
        call = self.procedures
        for m in met:
            if m in call:
                call = call[m]
            else:
                if '__proxy__' in call:
                   if '__subname__' in call:
                       call['__subname__'].append(m)
                   else:
                       call = {'__proxy__': call['__proxy__'], '__subname__': [m]}
                else:
                   call = {'__name__': m}
        if '__call__' in call:
            func = call['__call__']
        elif '__proxy__' in call:
            func = call['__proxy__'].service(call['__subname__'])
        else:
            call_error(-32600, u'Procedure %s not found' % call['__name__'])
            return out
        if ('__logg__' in call):
            logg = call['__logg__']
        if ('__auth__' in call) and ('auth' in inp):
            try:
                auth_res=call['__auth__'](inp['auth'])
            except StandardError:
                call_error(-32801, u'Authentication error')
                return out
        else:
            call_error(-32603, u'Missing authentication info')
            return out
        if auth_res:
            logg({'event': "auth succes", 'value': auth_res})
        else:
            call_error(-32701, u'Authentication failed: %s' % auth_res)
            return out
        try:
            out['result'] = func(par)
        except StandardError as ecx:
            call_error(-32800, u'An exception in called procedure:' + unicode(ecx)) # + format_exc())
            return out
        logg({'event': "call succes", 'value': out})
        return out
    def JPCS(self, inps):
        try:
            in_json = jloads(inps)
        except ValueError as e:
            in_json = None
            out_json = set_error({}, -32602, u'JSON Parse Error ' + str(e))
        if in_json != None:
            out_json = self.JPC(in_json)
        outs = jdumps(out_json)
        return outs
    def http_header_string_json(self):
        h = 'Content-type: application/json\n\n'
        return h
    def JPC_filter(self, json_string=''):
        if not json_string:
           json_string = sys.stdin.read()
        out_string = self.JPCS(json_string.decode('utf-8'))
        sys.stdout.write(out_string.encode('utf-8'))
    def JPC_CGI(self):
        form = cgi.FieldStorage()
        self.logg({'event': "CGI access", 'value': form})
        json_string = form.getfirst('json', '')
        out_string = self.JPCS(json_string.decode('utf-8'))
        json_response = self.http_header_string_json() + out_string.encode('utf-8')
        print json_response
    def main(self, name):
         if name == '__main__':
             if 'GATEWAY_INTERFACE' in os.environ:
                 self.JPC_CGI()
             else:
                 stdinp = sys.stdin.read()
                 if stdinp:
                     self.JPC_filter(stdinp)
                 else:
                     raise RuntimeError('Unrecognized context of call')
             return None, None
         else:
             return self.JPC, self.JPCS
if __name__ == '__main__':
    jpc = Server(tests=True)
    JPC, JPCS = jpc.main(__name__)