#!/usr/bin/env python
# -*- coding: utf-8 -*-

import re
import kaliope_patterns as patterns
__version__ = '1.0.20070709'
class Hyphenator:
    def __init__(self, patterns, exceptions=u''):
        self.tree = {}
        for pattern in patterns.split():
            self._insert_pattern(pattern)
        self.exceptions = {}
        for ex in exceptions.split():
            key = ex.replace('-', '')
            value = [0] + [int(h == '-') for h in re.split(r"[^\-]", ex)]
            self.exceptions[key] = value
    def _insert_pattern(self, pattern):
        chars = re.sub('[0-9]', '', pattern)
        points = [int(d or 0) for d in re.split(u'[^0-9]', pattern)]
        t = self.tree
        for c in chars:
            if c not in t:
                t[c] = {}
            t = t[c]
        t[None] = {}
        t[None]['points'] = points
        t[None]['pattern'] = pattern
    def hyphenate_word(self, word):
        return self.word_info(word)['pieces']
    def word_info(self, word):
        used = []
        algorithm = None
        wl = word.lower()
        if len(word) <= 4:
            algorithm = 'short'
            pieces = [word]
        elif wl in self.exceptions:
            points = self.exceptions[wl]
            pieces = self.word_points_to_pieces(word, points)
            algorithm = 'exception'
        else:
            work = '.'  +  wl + '.'
            points = [0] * (len(work) + 1)
            for i in range(len(work)):
                t = self.tree
                for c in work[i:]:
                    if c in t:
                        t = t[c]
                        if None in t:
                            p = t[None]['points']
                            for j in range(len(p)):
                                points[i+j] = max(points[i+j], p[j])
                            used.append(t[None]['pattern'])
                    else:
                        break
            points[1] = points[2] = points[-2] = points[-3] = 0
            algorithm = 'patterns'
            pieces = self.word_points_to_pieces(word, points)
        res = {
            'word': word,
            'lower': wl,
            'pieces': pieces,
            'algorithm': algorithm,
        }
        if res['algorithm'] == 'patterns':
            res['patterns'] = used
        return res
    def word_info_repr(self, wi):
        wl = wi['lower']
        s = u''
        s += '>>> ' + wl
        s += u'\n'
        s += self.repr_h(wi['pieces'])
        s += u'\n'
        if wi['algorithm'] in ['exception', 'short']:
            s += wi['algorithm']
        elif wi['algorithm'] == 'patterns':
            s += u' '.join(wi['patterns'])
        return s
    def word_points_to_pieces(self, word, points):
        pieces = ['']
        for c, p in zip(word, points[2:]):
            pieces[-1] += c
            if p % 2:
                pieces.append('')
        return pieces
    def repr_h(self, h):
        return u'-'.join(h)
pat = patterns.pl_classic_patterns
exc = patterns.pl_classic_exceptions
if __name__ == '__main__':
    h = Hyphenator(patterns.w05_patterns, patterns.w05_exceptions)
    li = u'''a bądźmy troszeczkę poważni
    abra jajko hyphenation konstantynopolitańczykowianeczka bynajmniej
bezach
bezami
bynajmniej
gdzieniegdzie
inaczej
nadal
nigdy
nigdzie
niechże
niechby
owszem
półach
półami
półek
podówczas
przynajmniej
skądinąd
trójach
trójami
trójek
    '''
    liX = u'''
ćwierćinteligent
'''
    wil = []
    hy = h.hyphenate_word
    wl = li.split()
    for w in li.split():
        wi = h.word_info(w)
        wil.append(h.word_info_repr(wi))
        pass
    print ('\n' + '-' * 40 + '\n').join(wil)
    if 0:
        import sys
        if len(sys.argv) > 1:
            for word in sys.argv[1:]:
                print '-'.join(hyphenate_word(word))
        else:
            import doctest
            doctest.testmod(verbose=True)