#!/usr/bin/env python
# -*- coding: utf-8 -*-

import gtk
import os
import re
import string
import braille_encodings as be
def event_to_dict(event):
    d = {}
    if event.state & gtk.gdk.CONTROL_MASK:
        d['ctrl'] = True
    if event.state & gtk.gdk.MOD1_MASK:
        d['left_alt'] = True
    if event.state & gtk.gdk.SHIFT_MASK:
        d['shift'] = True
    d['keyname'] = gtk.gdk.keyval_name(event.keyval)
    d['keyval'] = event.keyval
    d['string'] = event.string
    d['is_modifier'] = event.is_modifier
    d['state'] = int(event.state)
    return d
def mk_process_braille():
    priv = {
        'pressed': {},
        'collected': {}, 
    }
    def group_init():
        priv['pressed'] = {}
        priv['collected'] = {}
    def group_pressed(d):
        priv['pressed'][d['keyname']] = d
        priv['collected'][d['keyname']] = d
    def group_released(d):
        try:
            del priv['pressed'][d['keyname']]
        except KeyError:
            pass
        if len(priv['pressed']) == 0:
            res = {'done': True, 'group': priv['collected']}
            group_init()
        else:
            res = {'done': False, 'group': priv['collected']}
        return res
    def keys_to_braille(kk):
        return be.d2u_dict[keys_to_dots(kk)]
    def keys_to_dots(kk):
        kk = [k.lower() for k in kk]
        s = ''
        if 'e' in kk: s += '1'
        if 'w' in kk: s += '2'
        if 'q' in kk: s += '3'
        if 'i' in kk: s += '4'
        if 'o' in kk: s += '5'
        if 'p' in kk: s += '6'
        return s
    def is_in_group(d):
        return d['keyname'].lower() in ['q', 'w', 'e', 'i', 'o', 'p']
    def with_modifier(d):
        return 'ctrl' in d or 'left_alt' in d or 'shift' in d
    def process_key_press_event(event):
        d = event_to_dict(event)
        program = []
        if is_in_group(d):
            group_pressed(d)
            done = True
        elif d['keyname'] in ['space']:
            program.append(['insert', keys_to_braille({})])
            done = True
        elif d['keyname'] in ['Return']:
            done = False
        elif d['string'] in list(string.printable) or d['keyname'] in ['Tab']:
            done = True
        else:
            done = False
        res = {
            'program': program,
            'done': done,
        }
        return res
    def process_key_release_event(event):
        d = event_to_dict(event)
        program = []
        if is_in_group(d):
            r = group_released(d)
            if r['done']:
                program.append(['insert', keys_to_braille(r['group'])])
            done = True
        else:
            done = True
        res = {
            'program': program,
            'done': done,
        }
        return res
    res = {
        'key_press': process_key_press_event,
        'key_release': process_key_release_event,
    }
    return res
def mk_process_ascii_braille():
    def process_key_press_event(event):
        d = event_to_dict(event)
        program = []
        if d['string'] in list(string.printable) or d['keyname'] in ['Tab']:
            if be.is_ascii_braille(d['string']):
                program.append(['insert', d['string'].upper()])
                done = True
            else:
                done = True
        else:
            done = False
        res = {
            'program': program,
            'done': done,
        }
        return res
    def process_key_release_event(event):
        res = {
            'program': [],
            'done': False,
        }
        return res
    res = {
        'key_press': process_key_press_event,
        'key_release': process_key_release_event,
    }
    return res
def mk_process_transparent():
    def process_key_press_event(event):
        res = {
            'program': [],
            'done': False,
        }
        return res
    def process_key_release_event(event):
        res = {
            'program': [],
            'done': False,
        }
        return res
    res = {
        'key_press': process_key_press_event,
        'key_release': process_key_release_event,
    }
    return res
process_braille = mk_process_braille()
process_ascii_braille = mk_process_ascii_braille()
process_transparent = mk_process_transparent()