#!/usr/bin/env python
# -*- coding: utf-8 -*-

import math
import transform
tracing = True
tracing = False
def trace(*ar):
    if tracing:
        print ' '.join([str(e) for e in ar])
class Sensor(object):
    def __init__(self, par):
        area = par['area']
        ctm = par['ctm']
        self.on = par['on']
        self.id = par['id']
        self.ctm = ctm
        m = ctm
        self.ctmi = transform.Transform().transform((m[0], m[1], m[2], m[3], m[4], m[5]))
        self.ctmi.invert()
        self.click_point = None
        self.state = 'still'
        self.area = area
    def tp_gen(self, m, p):
        x, y = p
        return m.transform_point(x, y)
    def u2d(self, x, y=None):
        if y is None:
            x, y = x
        return self.tp_gen(self.ctm, (x, y))
    def d2u(self, x, y=None):
        if y is None:
            x, y = x
        return self.tp_gen(self.ctmi, (x, y))
    def inside(self, point):
        return False
    def button_press_event(self, x, y, button):
        if self.inside((x, y)) and button == 1:
            trace('click inside, device:', (x, y), 'user:', self.d2u(x, y))
            if self.click_point is not None:
                self.click_point = None
                trace('double click')
            else:
                self.click_point = (x, y)
                self.on_press(self.d2u(x, y))
        return True
    def button_release_event(self, x, y, button):
        trace(self, 'release, click point:', self.click_point)
        if button == 1:
            if self.click_point is None:
                pass
            else:
                self.on_release(self.d2u(x, y))
                self.click_point = None
        return True
    def motion_notify_event(self, x, y, first_button):
        if first_button:
            if self.click_point is None:
                pass
            else:
                trace('i am device:', (x, y), 'user:', self.d2u(x, y))
                self.on_drag(self.d2u(x, y))
                self.click_point = (x, y)
        else:
            if self.inside((x, y)):
                self.on_over(self.d2u(x, y))
            else:
                self.on_outside(self.d2u(x, y))
        return True
    def on_press(self, point):
        self.state = 'press'
        self.area.redraw()
        trace('press user:', point)
    def on_release(self, point):
        self.state = 'release'
        self.area.redraw()
        trace('release user:', point)
    def on_drag(self, point):
        self.state = 'drag'
    def on_over(self, point):
        self.state = 'over'
        self.area.redraw()
        trace('over')
    def on_outside(self, point):
        self.state = 'still'
        self.area.redraw()
        trace('outside')
    def dz(self, point):
        x0, y0 = self.d2u(self.click_point)
        x1, y1 = point
        dx = x1 - x0
        dy = y1 - y0
        return dx, dy
class RoundSensor(Sensor):
    def __init__(self, par):
        area = par['area']
        ctm = par['ctm']
        center = par['center']
        radius = par['radius']
        self.on = par['on']
        self.id = par['id']
        self.xc, self.yc = center
        super(RoundSensor, self).__init__(par)
        self.type = 'round'
        self.center = center
        self.radius = radius
        self.state = 'still'
    def json(self):
        return (self.type, self.center, self.radius)
    def __repr__(self):
        return 'sensor' + repr((self.type, self.center, self.radius, self.ctm))
    def inside(self, point):
        xc, yc = self.center
        xp, yp = self.d2u(point)
        dx = xp - xc
        dy = yp - yc
        dist = math.sqrt(dx * dx + dy * dy)
        res = dist < self.radius
        if res: trace('inside; users space', (xp, yp), 'close to', (xc, yc))
        return res
    def on_drag(self, point):
        xc, yc = self.center
        self.dx, self.dy = self.dz(point)
        self.center = xc + self.dx, yc + self.dy
        trace("(user's space) drag by", (self.dx, self.dy))
        onc = {
            'oldx': xc,
            'oldy': yc,
            'dx': self.dx,
            'dy': self.dy,
            'xc': self.center[0],
            'yc': self.center[1],
        }
        onc['id'] = self.id
        self.xc, self.yc = onc['xc'], onc['yc']
        self.on['changed'](onc)
        self.state = 'drag'
    def draw(self, context):
        color_still = (5, 101, 198, 12)
        color_press = (200, 51, 8, 42)
        color_over = (100, 151, 8, 42)
        self.color = {
            'still': color_still,
            'over': color_over,
            'press': color_press,
            'drag': color_press,
            'release': color_over,
            }
        color = self.color[self.state]
        r, g, b, a = color
        r = float(r)
        g = float(g)
        b = float(b)
        a = float(a)
        context.save()
        radius = self.radius
        xc, yc = self.center
        context.set_source_rgba(r/255.0, g/255.0, b/255.0, a/255)
        context.new_path()
        context.arc(xc, yc, radius, 0.0, 360.0)
        context.close_path()
        context.fill()
        context.restore()
        color = 5, 10, 19, 120
        r, g, b, a = color
        r = float(r)
        g = float(g)
        b = float(b)
        a = float(a)
        context.save()
        radius = 1
        xc, yc = self.center
        context.set_source_rgba(r/255.0, g/255.0, b/255.0, a/255)
        context.new_path()
        context.arc(xc, yc, radius, 0.0, 360.0)
        context.close_path()
        context.fill()
        context.restore()
if __name__ == "__main__":
    def main():
        import gtk
        import cairo
        import drawing
        window = gtk.Window(gtk.WINDOW_TOPLEVEL)
        window.set_name("Drawable")
        vbox = gtk.VBox(False, 0)
        window.add(vbox)
        vbox.show()
        window.connect("destroy", lambda w: gtk.main_quit())
        drawing_area = drawing.Whiteboard()
        drawing_area.set_size_request(500, 500)
        drawing_area.show()
        center = 200, 200
        radius = 20
        ctm = cairo.Matrix()
        ctm.translate(0, 500)
        ctm.scale(1, -1)
        sensor = RoundSensor(drawing_area, ctm, center, radius)
        drawing_area.sensors.append(sensor)
        vbox.pack_start(drawing_area, True, True, 0)
        button = gtk.Button("Quit")
        vbox.pack_start(button, False, False, 0)
        button.connect_object("clicked", lambda w: w.destroy(), window)
        button.show()
        status = drawing.Statusbar()
        vbox.pack_start(status, False, False, 0)
        status.show()
        drawing_area.status = status
        window.set_position(gtk.WIN_POS_CENTER)
        window.show()
        gtk.main()
        return 0
    main()