#!/usr/bin/env python
# -*- coding: utf-8 -*-

import pygtk
pygtk.require('2.0')
import gtk
import cairo
class DrawingArea(gtk.DrawingArea):
    def __init__(self):
        gtk.DrawingArea.__init__(self)
        self.connect("expose_event", self.expose_event)
        self.connect("size-allocate", self.size_allocate_event)
    def size_allocate_event(self, widget, event):
        pass
    def expose_event(self, widget, event):
        par = {
            'clip': {
                'x': event.area.x,
                'y': event.area.y,
                'w': event.area.width,
                'h': event.area.height,
                }
        }
        self.prepare_context_and_vieport(par)
        self.draw()
        return False
    def prepare_context_and_vieport(self, par):
        context = self.window.cairo_create()
        if 'clip' in par:
            cl = par['clip']
            context.rectangle(cl['x'], cl['y'],
                cl['w'], cl['h'])
            context.clip()
        self.context = context
        self.draw_rect = self.get_allocation()
        self.draw_x = self.draw_rect.x
        self.draw_y = self.draw_rect.y
        self.draw_width = self.draw_rect.width
        self.draw_height = self.draw_rect.height
    def draw(self):
        pass
    def redraw(self):
        if self.window:
            alloc = self.get_allocation()
            rect = gtk.gdk.Rectangle(0, 0, alloc.width, alloc.height)
            self.window.invalidate_rect(rect, True)
            self.window.process_updates(True)
class MotionArea(DrawingArea):
    def __init__(self):
        super(MotionArea, self).__init__()
        self.set_events(self.get_events()
            | gtk.gdk.BUTTON_PRESS_MASK
            | gtk.gdk.BUTTON_RELEASE_MASK
            | gtk.gdk.POINTER_MOTION_MASK
            | gtk.gdk.POINTER_MOTION_HINT_MASK)
        self.connect("motion_notify_event", self.motion_notify_event)
        self.connect("button_press_event", self.button_press_event)
        self.connect("button_release_event", self.button_release_event)
        self.cur_x = None
        self.cur_y = None
        self.cur_button = None
        self.cur_state = None
        if False:
            self.add_events(gtk.gdk.KEY_PRESS_MASK)
            self.add_events(gtk.gdk.KEY_RELEASE_MASK)
            self.connect("key_press_event", self.key_press_event)
            self.connect("key_release_event", self.key_release_event)
            self.set_flags(gtk.CAN_FOCUS)
            self.grab_focus()
    def button_press_event(self, widget, event):
        self.cur_x = event.x
        self.cur_y = event.y
        self.cur_button = event.button
        return True
    def button_release_event(self, widget, event):
        self.cur_x = event.x
        self.cur_y = event.y
        self.cur_button = event.button
        return True
    def motion_notify_event(self, widget, event):
        if event.is_hint:
            x, y, state = event.window.get_pointer()
        else:
            x = event.x
            y = event.y
            state = event.state
        self.cur_x = x
        self.cur_y = y
        self.cur_state = event.state
        return True
    if False:
        def key_press_event(self, widget, event):
            return False
        def key_release_event(self, widget, event):
            return False
class SensorBoard(MotionArea):
    def __init__(self):
        super(SensorBoard, self).__init__()
        self.unblock_sensors()
        self.clear_sensors()
        self.show_sensors = True
    def draw(self):
        super(SensorBoard, self).draw()
        if self.show_sensors:
            for s in self.sensors:
                if 'draw' in dir(s):
                    s.draw(self.context)
    def clear_sensors(self):
        if not self.blocked:
            self.sensors = []
            self.sensors_dict = {}
    def add_sensor(self, s):
        if not self.blocked:
            self.sensors.append(s)
            self.sensors_dict[s.id] = s
    def block_sensors(self):
        self.blocked = True
    def unblock_sensors(self):
        self.blocked = False
    def button_press_event(self, widget, event):
        super(SensorBoard, self).button_press_event(widget, event)
        for s in self.sensors:
            s.button_press_event(self.cur_x, self.cur_y, self.cur_button)
        return True
    def button_release_event(self, widget, event):
        super(SensorBoard, self).button_release_event(widget, event)
        for s in self.sensors:
            s.button_release_event(self.cur_x, self.cur_y, self.cur_button)
        return True
    def motion_notify_event(self, widget, event):
        super(SensorBoard, self).motion_notify_event(widget, event)
        for s in self.sensors:
            first_button = self.cur_state & gtk.gdk.BUTTON1_MASK
            s.motion_notify_event(self.cur_x, self.cur_y, first_button)
        return True
default_config = {
    'a': {
        'bg': ['bytes', [0, 0, 0]],
    },
    'on': {},
    'm': {}
}
def eval(ev, par):
    a = {}
    try:
        a.update(default_config['a'])
    except:
        pass
    try:
        a.update(par['a'])
    except:
        pass
    w = SensorBoard()
    w.show()
    def on(ev, f):
        w.connect(ev, f, w)
    try:
        ons = par['on']
        for k in ons:
            on(k, ons[k])
    except:
        pass
    def read(name):
        w.read(name)
        return w
    def redraw():
        w.redraw()
        return w
    res = {
        'widget': w,
        'on': on,
        'm': {
            'redraw': redraw,
            },
    }
    return res
if __name__ == "__main__":
    pass