#!/usr/bin/env python
# -*- coding: utf-8 -*-

__version__ = '0.93'
__date__= '10.03.2014'
import sys
import os
import re
import base64
import jpc
import ConfigParser
def read_config(fname): # read config file (server and authentication info)
    config = ConfigParser.RawConfigParser()
    config.read(fname)
    conf={}
    for sec in config.sections():
        conf[sec]={}
        for k,v in config.items(sec):
            conf[sec][k]=v
        if not ('url' in conf[sec] and 'func' in conf[sec]):
            print "Missing JPC URL or function name in config file."
            exit(1)
        print 'Activating "%s" service.' % (sec)
    return conf
def register_JPCs(conf):
    for srv,cnf in conf.items():
        auth={}
        for k in ('usern', 'user', 'passwd', 'token', 'salt'):
            if k in cnf:
                auth[k]=cnf[k]
        cnf['call']=jpc.Client(cnf['func'], cnf['url'], 'http', Auth=auth)
    return conf
def check_errors(res):
     if 'error' in res:
         print "Error in processing: ", res['error']
         exit(1)
def check_files(fname):
    bname = re.sub(r'\.[a-zA-Z0-9]+$', '', fname)
    ifn = bname + '.jpi'
    ofn = bname + '.jpo'
    if not os.access(ifn, os.R_OK):
        print "Can't find input file '%s'." % (ifn)
        exit(1)
    return ifn,ofn
def read_jpi_file(ifn):
    print 'Processing "%s".' % (ifn)
    with open(ifn, 'r') as infile:
        auxstr=infile.read().decode('utf-8')
    return auxstr
def write_jpo_file(ofn, oilist):
    oistr="%\n".join(oilist)
    oistr="% TeXJPC output info\n" +  oistr + "%\n%\n\\endinput"
    with open(ofn, 'w') as ofile:
        ofile.write(oistr)
    print 'File "%s" saved.' % (ofn)
def write_decoded_pdf(ofn,pdfstr):
    with open(ofn, 'wb') as pdf:
        pdf.write(base64.b64decode(pdfstr))
def write_log_file(ofn,logstr):
    with open(ofn, 'wb') as log:
        log.write(logstr)
def write_pdf_file(ifn, res): # write a PDF file from JSON string
    ofn=re.sub(r'\.[a-zA-Z0-9]+$', '', ifn) + '.pdf'
    if 'contents' in res:
        write_decoded_pdf(ofn,res['contents'])
    print 'File "%s" saved.' % (ofn)
"""
   jpcaux_split -- converts multiline string with block structure:
       %JPCBEGIN <service name>: <block name>
       <contents>
       %JPCEND <service name>: <block name>
   into dictionary of lists of tuples:
       {<service name>: [ (<block name>, <contents>), ...], ...}
"""
def jpcaux_split(auxstr, conf):
    infos = re.split("\n*%JPC(BEGIN [a-z]+: [A-Z ]+)\n|\n*%JPC(END [a-z]+: [A-Z ]+)\n", auxstr)
    infblocks = {}
    srv=''
    typ=''
    for block in infos:
        if block:
            m = re.match("^(BEGIN|END) ([a-z]+): ([A-Z ]+)$",block)
            if m:
                be, csrv, ctyp = m.group(1, 2, 3)
                if be=='BEGIN':
                    if csrv in conf:
                        srv=csrv
                        typ=ctyp
                        if srv not in infblocks:
                            infblocks[srv]=[]
                    else:
                        if csrv!='jpc':
                            print "Unconfigured service '%s' -- ignoring." % (csrv)
                        typ=''
                elif be=='END':
                    if srv!=csrv or typ!=ctyp:
                        if typ!='':
                            print "Unpaired tags in '*.jpi' file."
                    srv=''
                    typ=''
            else:
                if srv and typ:
                    infblocks[srv].append((typ,block))
    if not infblocks:
        print "No info blocks recognized in '*.jpi' file."
    return infblocks
"""
   jpc_json_structuralize -- converts structure generated by jpcaux_split:
       {<service name>: [ (<block name>, <contents>), ...], ...}
   into dictionary of 'object dictionares' -- every with three keys:
       {<service name>: {'global': <global>, 'objects': <objects>, 'info': <infos>}, ...}
   where:
       <global> -- dictionary with keys: 'info', 'params', 'prolog'
       <objects> -- list of dictionaries with keys: 'info', 'params', 'prolog' and 'value'
       <infos>  -- dictionary with keys 'global' and 'objects' which contains
          information extracted from key 'info' of the above object
"""
def jpc_json_structuralize(blocks):
    struct={}
    for srv,block in blocks.items():
        glob_infs={}
        glob_pars={}
        glob_prol=''
        objs=[]
        prol=''; val=''; pars={}; infs={}
        for ki,vi in block:
            if ki=='GLOBAL CONF':
                glob_infs=aux_params_to_dict(vi)
            elif ki=='GLOBAL PARS':
                glob_pars=aux_params_to_dict(vi)
            elif ki=='GLOBAL PROLOG':
                glob_prol=aux_to_text(vi)
            elif ki=='OBJECT PARS':
                pars=aux_params_to_dict(vi)
            elif ki=='OBJECT PROLOG':
                prol=aux_to_text(vi)
            elif ki=='OBJECT VALUE':
                val=aux_to_text(vi)
            elif ki=='OBJECT INFO':
                infs=aux_params_to_dict(vi)
                objs.append({'info': infs, 'params': pars, 'prolog': prol, 'value': val})
                prol=''; val=''; pars={}; infs={}
            else:
                print "Unrecognized block '%s' in '*.jpi' file -- ignoring." % (ki)
        glob={'info': glob_infs, 'params': glob_pars, 'prolog': glob_prol}
        infsd={'global': glob['info'], 'objects': [o['info'] for o in objs]}
        struct[srv]={'global': glob, 'objects': objs, 'info': infsd}
    return struct
def aux_params_to_dict(t):
    inf={}
    infls=re.split("([a-z_]+)=([^;]*);", t)
    k=''
    for i in infls:
        if k:
            try:
                i=int(i)
            except:
                try:
                    i=float(i)
                except:
                    pass
            inf[k]=i
            k=''
        else:
            if i.strip():
                k=i
    return inf
def aux_to_text(t):
    t=t.replace('^^M','\n')
    return t.rstrip()
def object_info_to_aux(srv,info):
    if 'label' in info:
        lab = info['label']
    elif 'lab' in info:
        lab = info['lab']
    elif 'filename' in info:
        lab = info['filename']
    elif 'fname' in info:
        lab = info['fname']
    infl=[]
    obj_to_aux(infl, info, '')
    infstr="\\csdef{jpc_%s_%s}{%s}" % (srv, lab, " ".join(infl))
    return infstr
def obj_to_aux(infl, obj,pref):
    for ik,iv in obj.items():
        k=pref + ik.replace("_","")
        if isinstance(iv,(int,float,long)):
            infl.append("\\defparnum{%s}{%s}" % (k, iv))
        elif isinstance(iv,basestring):
            infl.append("\\defpargen{%s}{%s}" % (k, iv))
        elif isinstance(iv,list):
            if len(iv)==2 and iv[1] in ('cm', 'mm', 'bp', 'in', 'pt', 'dd', 'sp'):
                 infl.append("\\defpardim{%s}{%s%s}" % (k, iv[0], iv[1]))
            elif len(iv)==2 and isinstance(iv[0], list) and len(iv[0])==2:
                 infl.append("\\defparpoint{%s}{%s%s}{%s%s}" % (k,
                    iv[0][0], iv[0][1], iv[1][0], iv[1][1]))
            else:
                obj_to_aux(infl, dict(((i,iv[i]) for i in range(len(iv)))),k+'_')
        elif isinstance(iv,dict):
            obj_to_aux(infl, iv,k+'_')
        else:
            infl.append("\\defparoth{%s}{%s}" % (k, repr(iv)))