#!/usr/bin/env python
# -*- coding: utf-8 -*-

import math
no_numpy = True
if not no_numpy:
    import numpy
class Transform(object):
    def __init__(self):
        self.identity()
    def identity(self):
        self.matrix = (1, 0, 0, 1, 0, 0)
        return self
    def translate(self, tx, ty=None):
        tx, ty = (tx, ty) if ty is not None else tx
        self.transform((1, 0, 0, 1, tx, ty))
        return self
    def scale(self, sx, sy=None):
        sx, sy = (sx, sy) if sy is not None else (sx,)
        self.transform((sx, 0, 0, sy, 0, 0))
        return self
    def rotate(self, angle=None):
        self.transform((math.cos(angle), math.sin(angle),
            - math.sin(angle), math.cos(angle), 0, 0))
        return self
    def skew(self, angle_x=None, angle_y=None):
        if angle_y is None:
            angle_x, angle_y = angle_y
        self.transform((1, math.tan(angle_x), math.tan(angle_y), 1, 0, 0))
        return self
    def transform(self, m):
        self.matrix = self.multiply_matrices(m, self.matrix)
        return self
    def multiply_matrices(self, m1, m2):
        a1, b1, c1, d1, e1, f1 = m1
        a2, b2, c2, d2, e2, f2 = m2
        a = a1 * a2 + b1 * c2
        b = a1 * b2 + b1 * d2
        c = c1 * a2 + d1 * c2
        d = c1 * b2 + d1 * d2
        e = e1 * a2 + f1 * c2 + e2
        f = e1 * b2 + f1 * d2 + f2
        return a, b, c, d, e, f
    def transform_point(self, x, y):
        a, b, c, d, e, f = self.matrix
        x_new = a * x + c * y + e
        y_new = b * x + d * y + f
        return x_new, y_new
    def __add__(self, other):
        m1 = self.get_matrix()
        m2 = other.get_matrix()
        mm = self.multiply_matrices(m1, m2)
        return Transform().transform(mm)
    def __mul__(self, other):
        return self.__add__(other)
    def inverse(self):
        if no_numpy:
            pass
        else:
            nm = self.to_numpy_matrix()
            self.from_numpy_matrix(nm.I)
        return self
    invert = inverse
    def to_numpy_matrix(self):
        return self.tuple_to_numpy_matrix(self.matrix)
    def from_numpy_matrix(self, nm):
        self.matrix = self.numpy_matrix_to_tuple(nm)
        return self
    def tuple_to_numpy_matrix(self, t):
        a, b, c, d, e, f = t
        return numpy.matrix([
            [a, b, 0],
            [c, d, 0],
            [e, f, 1]
            ])
    def numpy_matrix_to_tuple(self, nm):
        return nm[0, 0], nm[0, 1], nm[1, 0], nm[1, 1], nm[2, 0], nm[2, 1]
    def get_matrix(self):
        return self.matrix
    def __repr__(self):
        return str(self.matrix)
    def svg(self, compact=False):
        def one(el):
            oper, arg = el
            sep = ', '
            res = oper + '(' + sep.join([unicode(e) for e in arg]) + ')'
            return res
        res = u' '.join([one(e) for e in self.seq])
        return res
    def uni(self, compact=False):
        return self.svg(compact)
if __name__ == "__main__":
    pass