#!/usr/bin/env python
# -*- coding: utf-8 -*-

import pygtk
pygtk.require('2.0')
import gtk
import cairo
import os
import sensor_area as sa
import sensor
import geometry
class Whiteboard(sa.SensorBoard):
    def __init__(self, filename=None):
        super(Whiteboard, self).__init__()
        self.positions = []
        self.log = False
        self.status = None
        self.svg = None
        self_svg_bg = None
        self.filename = filename
        self.if_extra = False
        self.viewport = None
        self.node = None
        def do_nothing():
            pass
        self.do_append_sensors = do_nothing
    def size_allocate_event(self, widget, event):
        par = {}
        self.prepare_context_and_vieport(par)
        self.draw()
        try:
            p = {'ob': self, 'widget': widget, 'event': event, 'node': self.node}
            self.myon['size_change'](p)
        except:
            pass
    def button_press_event(self, widget, event):
        super(Whiteboard, self).button_press_event(widget, event)
        if self.log:
            print 'press  ', self.cur_x, self.cur_y, self.cur_button
        self.show_status()
        return True
    def button_release_event(self, widget, event):
        super(Whiteboard, self).button_release_event(widget, event)
        if self.log:
            print 'release', self.cur_x, self.cur_y, self.cur_button
        self.show_status()
        try:
            p = {'ob': self, 'widget': widget, 'event': event, 'node': self.node}
            self.myon['button_release'](p)
        except:
            pass
        return True
    def motion_notify_event(self, widget, event):
        super(Whiteboard, self).motion_notify_event(widget, event)
        if self.log:
            def int2bin(n, count=24):
                """returns the binary of integer n, using count number of digits"""
                return "".join([str((n >> y) & 1) for y in range(count-1, -1, -1)])
            print 'motion ', self.cur_x, self.cur_y, int2bin(self.cur_state, 20)
        self.show_status()
        return True
    def add_pointX(self, command, x, y):
        self.positions.append((command, (x, y)))
        self.redraw()
    def replace_svg(self, s):
        self.filename = s
        self.draw()
    def set_svg(self, s):
        self.svg = s
    def set_viewport(self, v):
        self.viewport = v
    def set_layer(self, ln, b):
        if ln == 0:
            pass
        elif ln == 1:
            pass
        elif ln == 2:
            self.if_extra = b
        else:
            pass
    def draw(self):
        self.just_draw()
        sa.SensorBoard.draw(self)
    def just_draw(self):
        self.draw_background()
        if self.filename is not None:
            if self.svg is None:
                self.svg = open(self.filename).read()
            else:
                pass
            svg = rsvg.Handle(data=self.svg)
            if self.viewport is None:
                w, h, w2, h2 = svg.get_dimension_data() # gives document's declared size
                viewp = {
                    'x': 0, 'y': 0,
                    'w': w, 'h': h,
                }
                self.viewport = viewp
            else:
                viewp = self.viewport
            matrix = self.viewport_matrix(viewp)['matrix']
            context = self.context
            context.transform(matrix)
            if self.svg_bg is not None:
                context.rectangle(viewp['x'], viewp['y'], viewp['w'], viewp['h'])
                bg_value = self.svg_bg[1]
                context.set_source_rgb(*bg_value)
                context.fill()
            svg.render_cairo(context)
            if self.if_extra:
                dir = os.path.dirname(self.filename)
                self.extrafilename = os.path.join(dir, 'extra.svg')
                self.extrasvg = open(self.extrafilename).read()
                extrasvg = rsvg.Handle(data=self.extrasvg)
                extrasvg.render_cairo(context)
            matrix.invert()
            context.transform(matrix)
            self.do_append_sensors()
    def draw_background(self, color=None):
        context = self.context
        x = self.draw_x
        y = self.draw_y
        width = self.draw_width
        height = self.draw_height
        if color is not None:
            r, g, b, a = color
        else:
            r, g, b, a = (255, 255, 255, 255)
            r, g, b, a = (0, 0, 0, 255)
        r = float(r)
        g = float(g)
        b = float(b)
        a = float(a)
        context.set_source_rgba(r/255.0, g/255.0, b/255.0, a/255)
        context.rectangle(x, y, width, height)
        context.fill()
    def viewport_matrix(self, viewp):
        rect = self.get_allocation()
        res = viewport_matrix(viewp, {'w': rect.width, 'h': rect.height})
        return res
    def viewport_origin_matrix(self, viewp):
        rect = self.get_allocation()
        res = viewport_origin_matrix(viewp, {'w': rect.width, 'h': rect.height})
        return res
    def drawX(self):
        context = self.context
        x = self.draw_x
        y = self.draw_y
        width = self.draw_width
        height = self.draw_height
        r, g, b, a = (255, 255, 255, 255)
        r = float(r)
        g = float(g)
        b = float(b)
        a = float(a)
        context.set_source_rgba(r/255.0, g/255.0, b/255.0, a/255)
        context.rectangle(x, y, width, height)
        context.fill()
        self.draw_path()
        for p in self.positions:
            command, point = p
            xc, yc = point
            self.draw_point(command, xc, yc)
        sa.SensorBoard.draw(self)
    def draw_path(self):
        color = 225, 225, 00, 30
        r, g, b, a = color
        r = float(r)
        g = float(g)
        b = float(b)
        a = float(a)
        context = self.context
        context.save()
        radius = 3
        context.set_line_width(radius * 2)
        context.set_line_cap(cairo.LINE_CAP_ROUND)
        context.set_line_join(cairo.LINE_JOIN_ROUND)
        context.set_source_rgba(r/255.0, g/255.0, b/255.0, a/255)
        for i in range(len(self.positions)):
            command, point = self.positions[i]
            if command == 'b':
                self.context.move_to(*point)
            else:
                self.context.line_to(*point)
            if command == 'e':
                context.stroke()
        context.stroke()
        context.restore()
    def draw_point(self, command, xc, yc):
        context = self.context
        if command == 'b':
            color = 5, 101, 98, 142
        elif command == 'm':
            color = 225, 225, 00, 130
        elif command == 'e':
            color = 205, 101, 98, 202
        else:
            pass
        r, g, b, a = color
        r = float(r)
        g = float(g)
        b = float(b)
        a = float(a)
        context.save()
        radius = 3
        context.set_source_rgba(r/255.0, g/255.0, b/255.0, a/255)
        context.arc(xc, yc, radius, 0.0, 360.0)
        context.close_path()
        context.fill()
        context.restore()
    def show_status(self):
        if self.status:
            s = u''
            s += 'x = ' + str(self.cur_x) + ' y = ' + str(self.cur_y)
            if self.cur_state & gtk.gdk.BUTTON1_MASK:
                s += ' button = 1'
            self.status.change(s)
def viewport_matrix(viewp, ext):
    x = viewp['x']
    y = viewp['y']
    w = viewp['w']
    h = viewp['h']
    width = ext['w']
    height = ext['h']
    fw, fh = geometry.fit_rectangle(w, h, width, height, keep_small=False)
    scale = float(fw) / float(w)
    trans_x = (width - fw) / 2
    trans_y = (height - fh) / 2
    matrix = cairo.Matrix()
    cairo.Matrix.translate(matrix, trans_x, trans_y)
    cairo.Matrix.scale(matrix, scale, scale)
    res = {
        'matrix': matrix,
        'trans': (trans_x, trans_y),
        'scale': (scale, scale),
        'ext': ext, # w, h
    }
    return res
def viewport_origin_matrix(viewp, ext):
    x = viewp['x']
    y = viewp['y']
    w = viewp['w']
    h = viewp['h']
    width = ext['w']
    height = ext['h']
    vm = viewport_matrix(viewp, ext)
    matrix = vm['matrix']
    cairo.Matrix.translate(matrix, x, y)
    vm['matrix'] = matrix
    return vm
def area_to_vieport_origin_matrix(vinfo, viewport):
    matrix = cairo.Matrix()
    matrix.translate(0, vinfo['ext']['h'])
    matrix.translate(*vinfo['trans'])
    matrix.scale(*vinfo['scale'])
    matrix.scale(1, -1)
    matrix.translate(-viewport['x'], -viewport['y'])
    return matrix
default_config = {
    'a': {
        'bg': ['bytes', [0, 0, 0.5]],
        'svg_bg': None,
        'filename': 'svg/04/01_similar_grzebanie/similar.svg',
        'show_sensors': True,
    },
    'on': {},
    'm': {}
}
def eval(ev, par):
    a = {}
    try:
        a.update(default_config['a'])
    except:
        pass
    try:
        a.update(par['a'])
    except:
        pass
    if 'filename' in a:
        pass
        w = Whiteboard(filename=a['filename'])
    else:
        w = Whiteboard()
    w.show()
    if 'filename' in a:
        pass
    if 'svg_bg' in a:
        w.svg_bg = a['svg_bg']
    if 'show_sensors' in a:
        w.show_sensors = a['show_sensors']
    w.myon = {}
    def on(ev, f):
        if ev in ['changed', 'button_release', 'size_change']:
            w.myon[ev] = f
        else:
            w.connect(ev, f, w)
    if True:
        ons = par['on']
        for k in ons:
            on(k, ons[k])
    else:
        print 'oj, niedobrze...'
        pass
    def read(name):
        w.read(name)
        return w
    def set_viewport(v):
        w.set_viewport(v)
        return w
    def set_show_sensors(v):
        w.show_sensors = v
        return w
    def set_svg(s):
        w.set_svg(s)
        return w
    def viewport_origin_matrix(viewp):
        return w.viewport_origin_matrix(viewp)
    def viewport_matrix(viewp):
        return w.viewport_matrix(viewp)
    def redraw():
        w.redraw()
        return w
    def draw():
        w.draw()
        return w
    def just_draw():
        w.just_draw()
        return w
    w.sensor_state = {}
    def on_changed_sensor(par):
        vinfo = viewport_origin_matrix(w.viewport)
        matrix = area_to_vieport_origin_matrix(vinfo, w.viewport)
        matrix.invert()
        val = {}
        for s in w.sensors:
            x, y = matrix.transform_point(s.xc, s.yc)
            val[s.id] = x, y
        if 'changed' in w.myon:
            w.myon['changed'](val)
    def viewport2ctm(w):
        pass
    def append_sensor(s):
        if s['type'] == 'round':
            par = {}
            par['area'] = w
            if w.viewport is None:
                matrix = cairo.Matrix()
            else:
                vinfo = viewport_origin_matrix(w.viewport)
                matrix = vinfo['matrix']
            par['ctm'] = cairo.Matrix()
            cen = s['center']
            rad = s['radius']
            if 1:
                matrix = area_to_vieport_origin_matrix(vinfo, w.viewport)
                cen = matrix.transform_point(*s['center'])
                rad = matrix.transform_distance(s['radius'], 0)[0]
            par['center'] = cen
            par['radius'] = rad
            par['on'] = {'changed': on_changed_sensor}
            par['id'] = s['id']
            sob = sensor.RoundSensor(par)
            w.add_sensor(sob)
    def append_sensors(sl):
        def do_append():
            if isinstance(sl, (list, tuple)):
                for s in sl:
                    append_sensor(s)
            else:
                append_sensor(sl)
            w.do_append_sensors = do_nothing
        def do_nothing():
            pass
        w.do_append_sensors = do_append
    def clear_sensors():
        w.clear_sensors()
    def set_layer(ln, b):
        w.set_layer(ln, b)
    mdict = {}
    if 'm' in par:
        mdict.update(par['m'])
    mdict.update({
            'redraw': redraw,
            'draw': draw,
            'just_draw': just_draw,
            'append_sensor': append_sensor,
            'append_sensors': append_sensors,
            'clear_sensors': clear_sensors,
            'set_show_sensors': set_show_sensors,
            'set_viewport': set_viewport,
            'set_svg': set_svg,
            'set_layer': set_layer,
            'viewport_origin_matrix': viewport_origin_matrix,
            'viewport_matrix': viewport_matrix,
        })
    res = {
        'widget': w,
        'on': ons,
        'm': mdict,
    }
    w.node = res
    return res
if __name__ == "__main__":
    class Statusbar(gtk.Statusbar):
        def __init__(self):
            gtk.Statusbar.__init__(self)
            self.show()
        def change(self, s):
            context_description = u"the only one"
            self.context_id = self.get_context_id(context_description)
            message_id = self.push(self.context_id, s)
            return self
    def main():
        import sensor
        window = gtk.Window(gtk.WINDOW_TOPLEVEL)
        window.set_name("Drawable")
        vbox = gtk.VBox(False, 0)
        window.add(vbox)
        vbox.show()
        window.connect("destroy", lambda w: gtk.main_quit())
        drawing_area = Whiteboard(filename='svg/test-ala-svg-100.svg')
        drawing_area.set_size_request(500, 500)
        drawing_area.show()
        vbox.pack_start(drawing_area, True, True, 0)
        center = 200, 200
        radius = 20
        sensor = sensor.RoundSensor(drawing_area, cairo.Matrix(), center, radius)
        drawing_area.sensors.append(sensor)
        button = gtk.Button("Quit")
        vbox.pack_start(button, False, False, 0)
        button.connect_object("clicked", lambda w: w.destroy(), window)
        button.show()
        status = Statusbar()
        vbox.pack_start(status, False, False, 0)
        status.show()
        drawing_area.status = status
        window.set_position(gtk.WIN_POS_CENTER)
        window.show()
        gtk.main()
        return 0
    main()